function C=mtimes(X,Y)
% -- mtimes(X,Y)
%     Return the matrix multiplication product of inputs; i.e. X*Y.
%
%     For X and Y amat objects with compatible dimensions, 
%       X a N-by-m-by-n amat and Y a N-by-n-by-p amat return a
%       N-by-m-by-p amat object C such that
%          C(k)=X(k)*Y(k), for all k in 1...N
%
%     For X a N-by-m-by-n amat and Y a n-by-p matrix, [or X a m-by-n matrix and
%     Y a N-by-n-by-p amat,] return a N-by-m-by-p amat object C such that
%          C(k)=X(k)*Y, for all k in 1...N  
%     [or  C(k)=X*Y(k), for all k in 1...N  ]
%
%     For X a N-by-m-by-n amat and Y a N-by-1 or 1-by-N array, [or X a N-by-1 
%     or 1-by-N array and Y a N-by-m-by-n amat,] return a N-by-m-by-n amat 
%     object C such that
%          C(k)=X(k)*Y(k), for all k in 1...N  
%     [or  C(k)=X(k)*Y(k), for all k in 1...N  ]
%
%     For X a N-by-m-by-n amat and Y a scalar, [or X a scalar and Y a 
%     N-by-m-by-n amat,] return a N-by-m-by-n amat object C such that
%          C(k)=X(k)*Y, for all k in 1...N  
%     [or  C(k)=X*Y(k), for all k in 1...N  ]
%
  isXamat=isamat(X);isYamat=isamat(Y);
  assert( isXamat || isYamat , 'One of the parameters must be an <amat> object')
  if isscalar(X) || isscalar(Y), C=X.*Y;return;end % X or Y is constant (to improve performance)
  [getX,getY,N,dimX,dimY]=fc_amat.getVecs(X,Y);
  if all(dimX==[N,1,1]) || all(dimY==[N,1,1])
     C=X.*Y;return;
  end
  assert( dimX(3)==dimY(2) ,'Inner matrix dimensions must agree.');
  classname=class(getX(X,1,1).*getY(Y,1,1));
  C=amat(N,dimX(2),dimY(3),classname);
  if (~isreal(X) || ~isreal(Y)), C=complex(C);end
    
  for i=1:dimX(2)
    for j=1:dimY(3)
      for l=1:dimX(3)
        C.values(:,i,j)=C.values(:,i,j)+getX(X,i,l).*getY(Y,l,j);
      end
    end
  end
end


